/*
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gauges/RMI/RMIBandwidthGaugeMgr.java,v 1.4 2002/01/31 16:39:10 schmerl Exp $
 * $Log: RMIBandwidthGaugeMgr.java,v $
 * Revision 1.4  2002/01/31 16:39:10  schmerl
 * Added CVS header information
 *
 */
package edu.cmu.cs.able.gauges.RMI;
import java.rmi.*;
import java.util.*;

import java.net.*;
import java.io.*;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.RMI.*;

public class RMIBandwidthGaugeMgr extends RMIGaugeMgr {
    Hashtable gaugeProcessTable = new Hashtable();     
    private String sienaHost = null;
    /** Creates new RMIBandwidthGaugeMgr */
    public RMIBandwidthGaugeMgr(String bushost, String sienahost) {
        super(bushost);
        sienaHost = sienahost;        
        busHost = bushost;
        try {
            gaugeBus = new RMIGaugeReportingBus(bushost);
            gaugeMgrType = "edu.cmu.cs.able.gauges.RMI.RMIBandwidthGauge";
            gaugeMgrUID = Global.generateUID();
            String host = Initialization.startGaugeMgrService(this);
            gaugeBus.registerGaugeMgr(gaugeMgrUID, gaugeMgrType, host);
        }
        catch (Exception e) {}
    }

    /** Creates a new gauge as a separate process.
     * @param gauge The ID of the gauge to create.
     * @param setupParams The setup parameters to pass to the new gauge.
     * @param mappings The mappings to pass to the new gauge.
     * @return A gauge control that has the new ID and setup parameters.
     */    
    public GaugeControl createGauge(GaugeID gauge, StringPairVector setupParams, StringPairVector mappings) {
        String setup = "";
        for (int i = 0; i < setupParams.size()-1; i++) {
            setup = setup+setupParams.nameAt(i)+"="+setupParams.valueAt(i)+" ";
        }
        if ( setupParams.size() > 0) {
            setup = setup+setupParams.nameAt( setupParams.size()-1)+"="+setupParams.valueAt( setupParams.size()-1);
        }

        String uid = Global.generateUniqueID();
        gauge.uid = uid;    
        String cmd = "java "+gauge.gaugeType+" -gn "+gauge.gaugeName+" -gt "+gauge.gaugeType+" -mn "+gauge.modelName+" -mt "+gauge.modelType+" -uid "+uid+" -setup "+setup+" -senp "+sienaHost+" -rmib "+busHost;
        
        System.out.println(cmd);
        Process gaugeProcess = Global.exec(cmd);
        String host = null;
        GaugeID gaugeID = null;
        RMIGaugeHandle gh = null;
        int timeout = 20;
        try {
            host = gaugeBus.getGaugeHost(gauge);
            while (host == null && timeout > 0) {
                Thread.sleep(100);
                timeout--;
                host = gaugeBus.getGaugeHost(gauge);
            }
            System.err.println(host);
            gh = new RMIGaugeHandle(host);
            gaugeID = gh.getGaugeID();            
        }
        catch (Exception e) {}
        gaugeProcessTable.put(gauge.toString(), gaugeProcess);
        return gh;
    }

    /** Deletes the gauge process associated with the gauge ID.
     * @param gauge The gauge to be deleted.
     * @return Whether the process was destroyed successfully.
     */    
    public boolean deleteGauge(GaugeControl gauge) {
        Process gaugeProcess = (Process )gaugeProcessTable.get (gauge.getGaugeID ().toString());
        System.out.println(gauge.getGaugeID ().toString());
        if (gaugeProcess != null) {
            // Delete the gauge process
            try {
                System.out.println("try to kill the gauge process"+gaugeProcess.toString());
                gaugeProcess.destroy ();
                System.out.println("gauge process killed");
            }
            catch (Throwable t) {
                System.err.println ("Could not destroy gauge");
                t.printStackTrace ();
                return false;
            }
        }
        return true;
    }

    /** Returns true if the gauge type is edu.cmu.cs.able.gauges.RMI.RMIBandwidthGauge
     * @param gaugeType
     * @return
     */    
    public boolean managesType(String gaugeType) {
        return gaugeType.equals ("edu.cmu.cs.able.gauges.RMI.RMIBandwidthGauge");
    }
    
    /** Returns the parameters that can be used to configure the gauge,
     * as well as the values reported by the gauge, for a particular
     * gauge type.
     * @param gaugeType The type of the gauge about which to get information.
     * @param configParamsMeta Will fill this vector with [("Frequency", "int")]
     * @param valuesMeta Will fill this vector with [("bandwidth", "float")]
     * @return
     */    
    public boolean queryMetaInfo(String gaugeType, StringPairVector configParamsMeta, StringPairVector valuesMeta) {
        if (managesType (gaugeType)) {
            configParamsMeta.addElement ("Frequency", "int");
            valuesMeta.addElement ("bandwidth", "float");
            return true;
        }
        else {
            return false;
        }
    }
    
    
    public static void main(String args[]) {
        if (args.length < 2) {
            System.err.println("Eventbus host and/or siena host missing");
            return;
        }
        
        RMIBandwidthGaugeMgr theBandwidthGaugeMgr = new RMIBandwidthGaugeMgr(args[0], args[1]);
    }
}

