/*
 * SienaBandwidthGauge.java
 *
 * Created on November 13, 2001, 12:01 PM
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gauges/Siena/SienaBandwidthGauge.java,v 1.4 2002/02/20 20:41:30 schmerl Exp $
 * $Log: SienaBandwidthGauge.java,v $
 * Revision 1.4  2002/02/20 20:41:30  schmerl
 * 1. Fixed gauge deletion in the Siena version so that deletes don't timeout.
 * 2. Changed gauge implementation pattern so that gauge computation does not need to be included in both RMII and Siena versions.
 *
 * Revision 1.3  2002/01/31 16:39:11  schmerl
 * Added CVS header information
 *
 */

package edu.cmu.cs.able.gauges.Siena;
import edu.cmu.cs.able.gauges.RemosBandwidthGaugeImpl;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.probes.RemosProbe.*;
import java.net.*;
import java.util.*;
/** This class implements a BandwidthGauge, and uses RemosProbes
 * to determine the current bandwidth. It is assumed that a RemosProbe
 * is listening on the siena bus. The gauge type for this gauge is: <br>
 * BandwidthGauge = [<br>
 * &nbsp;&nbsp;type = edu.cmu.cs.able.gauges.Siena.SienaBandwidthGauge<br>
 * &nbsp;&nbsp;setup = [IP_addr1 : String = "localhost"; IP_addr2 = "localhost"; Frequency : long = 100]<br>
 * &nbsp;&nbsp;config = [Frequency : long = 100]<br>
 * &nbsp;&nbsp;values = [bandwidth : float]<br>
 * @author Bradley Schmerl
 * @version 1.1
 */
public class SienaBandwidthGauge extends edu.cmu.cs.able.gaugeInfrastructure.Siena.SienaGauge {

    /** The type of the BandwidthGauge (edu.cmu.cs.able.gauges.Siena.SienaBandwidthGauge)
     */    
    protected String gaugeType = "edu.cmu.cs.able.gauges.Siena.SienaBandwidthGauge";
    
    protected static String probeBus = null;
    RemosBandwidthGaugeImpl impl = null;

    /** Creates new SienaBandwidthGauge
     * @param gaugeID The ID of the gauge to create.
     * @param creatorID The ID of the gauge manager that is creating the gauge.
     * @param setupParams The value of the setup params, with items for IP_addr1, IP_addr2, and Frequency.
     * @param mappings The definition of the mapping between the bandwidth value and a model.
     */
    public SienaBandwidthGauge(GaugeID gaugeID, String creatorID, StringPairVector setupParams, StringPairVector mappings) {
        super (gaugeID, creatorID, setupParams, mappings);
       
        impl = new RemosBandwidthGaugeImpl (gaugeID, setupParams, mappings, gaugeBus, probeBus);
 
        int retries = 0;
        finalizeCreation (creatorID, impl.consistentlyCreated ());        
    }

    /** Configures the gauge.
     * @param configParams The configuration parameters for this gauge.
     * @return
     */
    public boolean configure(StringPairVector configParams) {
        return impl.configure (configParams);
    }
    
    /** Returns all the values that this gauge reports.
     * @param values Filled with an entry for bandwidth, with the value of the currentBW.
     * @return
     */
    public boolean queryAllValues(GaugeValueVector values) {
        return impl.queryAllValues (values);
    }
    
    /** Returns the state of this gauge.
     * @param setupParams The parameters that the gauge was setup with.
     * @param configParams The current configuration of the gauge.
     * @param mappings The current mappings of the gauge.
     * @return
     */
    public boolean queryState(StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        return impl.queryState (setupParams, configParams, mappings);
    }
    
    /** Returns the value of a particular value name
     * @param valueName The name of the value to return.
     * @return The gauge value
     */
    public String queryValue(String valueName) {
        return impl.queryValue (valueName);
    }
    
    /** A command-line interface to start the gauge.
     * @param args -gt <i>gauge type</i><br>
     * -gn <i>gauge name</i><br>
     * -mt <i>model type</i><br>
     * -mn <i>model name</i><br>
     * [-setup <i>name=value</i>]<br>
     * [-mappings <i>name=property</i>]<br>
     * -creator <i>creator ID</i><br>
     * -senp <i>siena port</i><br>
     * [-debug]
     */    
    public static void main(String[] args) {
           
        String creatorID = null;
        String gaugeName = null;
        String gaugeType = null;
        String modelName = null;
        String modelType = null;
        StringPairVector mappings = new StringPairVector ();
        StringPairVector setupParams = new StringPairVector ();
        boolean gotMappings = false;
        boolean gotSetup = false;
        // Process args
        boolean ok = true;
        for (int i = 0; i < args.length && ok; i++) {
            String tag = args [i];
            if (args [i].equals ("-creator")) {
                creatorID = args [++i];
            }
            else if (args [i].equals ("-gn")) {
                gaugeName = args [++i];
            }
            else if (args [i].equals ("-gt")) {
                gaugeType = args [++i];
            }
            else if (args [i].equals ("-mn")) {
                modelName = args [++i];
            }
            else if (args [i].equals ("-mt")) {
                modelType = args [++i];
            }
            else if (args [i].equals ("-setup")) {
                gotSetup = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String setup = args [i];
                    String name = setup.substring (0, setup.indexOf ('='));
                    String value = setup.substring (setup.indexOf ('=') + 1);
                    setupParams.addElement (name, value);
                    i++;
                } 
                i--;
            }
            else if (args [i].equals ("-mappings")) {
                gotMappings = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String mapping = args [i];
                    String value = mapping.substring (0, mapping.indexOf ('='));
                    String property = mapping.substring (mapping.indexOf ('=') + 1);
                    mappings.addElement (value, property);
                    i++;
                }
                i--;
            }
            else if (args [i].equals ("-senp")) {
                probeBus = args [++i];
                edu.cmu.cs.able.gaugeInfrastructure.Siena.Initialization.initSiena (probeBus);
            }
            else if (args [i].equals ("-debug")) {
                edu.cmu.cs.able.gaugeInfrastructure.util.Global.debugFlag = true;
            }
            else {
                ok = false;
            }            
        }
        
        if (!ok) {
            printUsageMessage ();
            System.exit (0);
        }
        
        GaugeID gid = new GaugeID ();
        gid.modelName = modelName;
        gid.modelType = modelType;
        gid.gaugeName = gaugeName;
        gid.gaugeType = gaugeType;
   
       
        SienaBandwidthGauge gauge = new SienaBandwidthGauge (gid, creatorID, setupParams, mappings);
       
        
    }
    
    private static void printUsageMessage () {
        System.out.println ("Usage: SienaBandwidthGauge -creator <creatorID> -mt <modelType> -mn <modelName> -gt <gaugeType> -gn <gaugeName> -setup <name=value> -mappings <value=property> -senp <sienaport>");
        
    }
    
    
}
