package edu.cmu.cs.able.probes.RemosProbe;

import java.net.*;
import java.io.*;
import java.util.*;

/**
 * ProbeManager is "behind" ProbeManageComm, but it is the object that manages
 * probes, ProbeManageComm takes care of the siena message communication so 
 * ProbeManager could focus on the work about probe management.
 */
public class ProbeManager {
	/** pmc takes care of the siena communication */
	ProbeManagerComm pmc;
	/** keeps the probes that have been created */
	Vector probeList;
	
	boolean debugf = true;
 
	/**
	 * Initialization.
	 * @param pmc the ProbemanagerComm that creates this manager
	 */
   	ProbeManager(ProbeManagerComm pmc) {
	   this.pmc = pmc;
	   probeList = new Vector(8);
   	};

	/** 
	 * Create a Remos Probe 
	 * @return the ID of the probe that is created 
 	 */
	public ProbeID createProbe() {
	   debugln("createProbe(): start");

	   Probe curProbe = new Probe(this);
	   curProbe.start();
	   /* TODO:
	    * Error process -- add a path for creation fail
	    */

	   probeList.addElement(curProbe);

	   debugln("createProbe(): done. element# " + probeList.size());

	   return curProbe.getID();
	}

	/** 
	 * Destory a Probe (either Flow or Topology Probe)
	 * @param prbID ID of the probe to be destroied
	 * @return true/false, i.e., whether the destroy is successful
	 */
   	public boolean destroyProbe(ProbeID prbID) {
	     debugln("destroyProbe(): start");

	     Probe prb = getProbeByID(prbID);
	 
	     if (prb == null) {
		System.err.println("Probe Not Found");
		return false;
	     }

	     prb.stopProbe();
	     // prb.destroy();
	     // prb.stop();
	     probeList.removeElement(prb);

	     debugln("destroyProbe(): done.");
	     return true;
   	};

	/**
	 * Search for the probe object in the probeList according to
	 * probe ID.
	 * @param prbID the ID of the probe that is searched for 
	 * @return the probe object that we find. If no object is found, null is return
	 */
	public Probe getProbeByID(ProbeID prbID) {

	   String id = prbID.toString();
	   Probe prb;

	   debugln("getProbeByID(): id = "+id);

	   int i = 0;
	   debugln("elements number: "+probeList.size());
	   /* TODO:
	    * The search algorithm is too naive, may incur performance overhead
	    */
	   while (i < probeList.size()) {
		if ((prb = (Probe)probeList.elementAt(i)) != null) {
		    debugln("element id"+prb.getID().toString());
		    if (prb.getID().toString().equals(id)) 
			return prb; 
		    else i++;
		}
	   }

	   return null;
	}

	/**
	 * Simply forward the probing information from Probe to ProbeManager
	 * @param prbID the ID of the probe that creates the error
	 * @param bw available bandwidth from the probe
	 */
	public void report(ProbeID prbID, double bw) {
	    pmc.sensed(prbID, bw);
	}

	/**
	 * Simply forward the error probe ID to ProbeManager
	 */
	public void reportError(ProbeID prbID) {
	     pmc.reportError(prbID);
	}

	/**
	 * Dump the debugging message
	 * @param s debugging message
	 */
	private void debugln(String s) {
	   System.out.println("RemosProbe.ProbeManager:"+s);
	}
}
